#!/usr/bin/env bash

# note, manual pre-req:
# parted
# mkpart btrfs $START_OF_FREE 100%
# mkfs.btrfs /dev/sda3

# --------
# -static-
# --------
# ---lxc adjustments---
_SUBUID="root:1000000:1000000000"
_SUBGID="root:1000000:1000000000\nlxd:1000000:1000000000"
# - - -

# ---ssh publics---
_KAYOS="ssh-ed25519 AAAAC3NzaC1lZDI1NTE5AAAAIMjlthGrGLciXD7c+aGr1RnSGTndIlLDDlUUw3kUdyYt tr4sh1fi"
_ROOBA="ssh-ed25519 AAAAC3NzaC1lZDI1NTE5AAAAIPU4tYfwpjvOV3nSY5jwfqJCPYuPf8c/8PnNUep5nFBT rooba@smoqueed.com"
# - - -

# ---color---
#GREEN="\e[0;32m"
#PURPL="\e[1;95m"
_FATAL="\e[1;31m"
#YELLO="\e[0;33m"
RESET="\e[0m"
# - - -

# ---misc---
LPLUS="\e[0;33m[\e[0;32m+\e[0;33m]\e[0m"
LFAIL="\e[0;33m[\e[1;95m-\e[0;33m]\e[0m"
# - - -
# --------------------------------------

# --------
# -letvar-
# --------
export _APT="apt-get"
export _AGET="$_APT -y install"
# --------------------------------------

# --------
# -funcs-
# --------
function getaptfast() {
	if ! type aria2c >/dev/null 2>&1; then
		sudo apt-get update
		sudo apt-get install -y aria2
	fi

	wget https://raw.githubusercontent.com/ilikenwf/apt-fast/master/apt-fast -O /usr/local/sbin/apt-fast
	chmod +x /usr/local/sbin/apt-fast
	if ! [[ -f /etc/apt-fast.conf ]]; then
		wget https://raw.githubusercontent.com/ilikenwf/apt-fast/master/apt-fast.conf -O /etc/apt-fast.conf
	fi
}
function apt_fast() {
	log "installing apt-fast..."
	if getaptfast; then
		return 0
	fi
	return 1
}
function apt_stage1() {
	lapt="$1"

	log "upating apt..."
	apt-get -y --fix-broken install >/dev/null ||
		fatal "apt b0rkd"
	$lapt update ||
		fatal "failed to update apt"

	log "running any pening upgrades..."
	$lapt -y full-upgrade ||
		fatal "failed to upgrade w/ apt"

	log "installing some base packages..."
	$lapt -y install git curl wget build-essential kitty-terminfo sudo cgroup-tools cgroupfs-mount ctop lxcfs libpam-cgroup ntpdate vnstat ncdu dialog ||
		fatal "failed to install basic packages"

}

function yeet_netplan() {
	lapt="$1"
	$lapt -y purge netplan.io ||
		return 1
	rm -vrf /usr/share/netplan
	rm -vrf /etc/netplan
}
function prov_interfaces() {
	log "provisioning /etc/network/interfaces..."
	mkdir -p /etc/network/ || fatal "failed to mkdir /etc/network"
	touch /etc/network/interfaces
	rm -vf /etc/network/interfaces 2 &>/dev/null
	echo -e "auto lo\n\tiface lo inet loopback\nauto $(def iface)\n\tiface $(def iface) inet dhcp" | tee /etc/network/interfaces
}
function def() {
	case "$*" in
	ipadr) /bin/ip route | grep default | awk '{print $3}' ;;
	iface) /bin/ip route get 8.8.8.8 | awk -- '{printf $5}' ;;
	*) fatal "what" ;;
	esac
}

function wrangle_dns() {
	systemctl --now disable systemd-resolved.service || return 1

	chattr -i /etc/resolv.conf 2 &>/dev/null

	rm -vf /etc/resolv.conf || return 1

	if dhclient "$(def iface)"; then return 0; fi

	err "unable to take control of DNS with DHCP..."
	log "falling back to '$(def ipadr)' ..."
	echo "nameserver $(def ipadr)" >/etc/resolv.conf || return 1

	# as long as we have working dns
	return 0
}

function net_ops() {
	lapt="$1"

	log "installing ifupdown2..."
	$lapt -y install ifupdown || return 1

	log "yeeting netplan..."
	yeet_netplan "$lapt" || return 1

	log "provisioning interfaces..."
	prov_interfaces || return 1

	log "taking control of DNS..."
	wrangle_dns || return 1
}
function clean_users_grps() {
	deluser --backup rooba
	deluser --backup kayos
	delgroup --system lxd
}
function clean_disk() {
	wipefs -a /dev/sda3
}
function clean_ops() {
	log "removing existing lxd..."
	snap remove lxd

	dialog --title "Run cleanup on users and groups?" --backtitle "Cleanup?" --yesno "Do you want to delete users kayos, rooba, and group LXD?" 7 60
	# 0 means yes
	# 1 means no
	# 255 means abort
	ans=$?
	case $ans in
	0) log "cleaning up..." && clean_users_grps ;;
	1) log "skipping user cleanup" ;;
	255) fatal "aborted" ;;
	esac

	dialog --title "Run cleanup on /dev/sda3?" --backtitle "Cleanup?" --yesno "Do you want to wipe /dev/sda3?" 7 60
	ans=$?
	case $ans in
	0) log "cleaning up..." && clean_disk ;;
	1) log "skipping disk cleanup" ;;
	255) fatal "aborted" ;;
	esac
}

function user_ops() {
	log "adding new users+lxd group..."
	uad="useradd --shell /bin/bash --create-home "
	$uad kayos || return 1
	$uad rooba || return 1
	groupadd -f --system lxd || true
	# ---------------------------
	usermod -G lxd -a kayos || return 1
	usermod -G lxd -a rooba || return 1
	sudo -u kayos mkdir -pv /home/kayos/.ssh
	sudo -u rooba mkdir -pv /home/rooba/.ssh
	echo "$_KAYOS" >>/home/kayos/.ssh/authorized_keys || return 1
	echo "$_ROOBA" >>/home/rooba/.ssh/authorized_keys || return 1
	_nopwd="   ALL=(ALL) NOPASSWD: ALL"
	echo -e "kayos${_nopwd}\nrooba${_nopwd}" >>/etc/sudoers || return 1
	service sudo restart 2 &>/dev/null || return 0
}
function lxc_ops() {
	echo -e "$_SUBUID" >>/etc/subuid || return 1
	echo -e "$_SUBGID" >>/etc/subgid || return 1

	snap install lxd || return 1
	snap switch lxd --channel latest/candidate || return 1
	snap refresh lxd || return 1

	# non destructive - if pre-existing it will fail
	lxd init --auto --storage-backend btrfs --storage-create-device /dev/sda3
}
function yeet_motd() {
	rm -vrf /etc/default/motd-news
	echo RU5BQkxFRD0wClVSTFM9IiIKV0FJVD0wCg== | base64 -d >/etc/default/motd-news
}
function log() {
	_date="\e[90m[$(date -u +'%H:%M:%S')]${RESET}"
	echo -e "${_date} ${LPLUS} $* ${RESET}"
}
function err() {
	_date="\e[90m[$(date -u +'%H:%M:%S')]${RESET}"
	echo -e "${_date} ${LFAIL} $* ${RESET}"
}
function fatal() {
	err "${_FATAL}FATAL${RESET}: $*"
	exit 1
}
# --------------------------------------

function main() {
	if [[ $UID != 0 ]]; then
		fatal "this script must be run as root."
	fi

	if apt_fast >/dev/null; then
		export _APT="apt-fast"
		export _AGET="$_APT -y install"
		log "using apt-fast"
	else err "apt-fast failed... continuing with apt-get"; fi

	log "--> apt stage1"
	apt_stage1 "$_APT"

	log "--> net ops"
	net_ops "$_APT" || fatal "net ops fail"

	clean_ops

	log "--> add users"
	user_ops || fatal "user ops fail"

	log "--> lxc stuff"
	lxc_ops || fatal "lxc/lxd fail"
}

main
